/* Autogenerated by png2wx.pl on Mon Jan  8 09:10:27 2018 */
#ifndef ICONS_H
#define ICONS_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_pypilot_blue;
extern wxBitmap *_img_pypilot_cyan;
extern wxBitmap *_img_pypilot_green;
extern wxBitmap *_img_pypilot_grey;
extern wxBitmap *_img_pypilot_red;
extern wxBitmap *_img_pypilot_yellow;

#endif /* ICONS_H */
